package net.wanji.web.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.vo.CrossDeviceStatusInfoOutVO;
import net.wanji.web.service.SituationDetectionService;
import net.wanji.web.vo.situationDetection.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.ws.rs.core.MediaType;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * 态势监测
 *
 * @author Kent HAN
 * @date 2022/10/26 13:31
 */
@Api(value = "SituationDetectionController", description = "状态检测")
@RequestMapping("/situationDetection")
@RestController
public class SituationDetectionController extends BaseController {

    @Autowired
    SituationDetectionService situationDetectionService;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @AspectLog(description = "优化记录", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "优化记录", notes = "优化记录", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/optRecords",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject optRecords(@RequestBody OptRecordsInVO optRecordsInVO) throws ParseException {
        String adCode = optRecordsInVO.getAdCode();
        String currentTimeStr = optRecordsInVO.getCurrentTime();
        Date adaptTime = sdf.parse(currentTimeStr);
        OptRecordsOutVO optRecordsOutVO = situationDetectionService.optRecords(adCode, adaptTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(optRecordsOutVO);
    }

    @AspectLog(description = "路口报警", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "路口报警", notes = "路口报警",
            response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossAlarm", produces = MediaType.APPLICATION_JSON)
    public JsonViewObject crossAlarm() {
        List<CrossAlarmOutVO> crossAlarmOutVOList = situationDetectionService.crossAlarm();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(crossAlarmOutVOList);
    }

    @AspectLog(description = "事件报警", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "事件报警", notes = "事件报警", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/eventAlarm",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject eventAlarm(@RequestBody EventAlarmInVO eventAlarmInVO) throws ParseException {
        String currentTimeStr = eventAlarmInVO.getCurrentTime();
        Date todayTime = sdf.parse(currentTimeStr);
        List<EventAlarmOutVO> eventAlarmOutVOList = situationDetectionService.eventAlarm(todayTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(eventAlarmOutVOList);
    }

    @AspectLog(description = "信号控制状态", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "信号控制状态", notes = "信号控制状态", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/controlHist",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject controlHist(@RequestBody ControlHistInVO controlHistInVO) throws ParseException {
        String crossId = controlHistInVO.getCrossId();
        String currentTime = controlHistInVO.getCurrentTime();
        List<ControlHistOutVO> controlHistOutVOList = situationDetectionService.controlHist(crossId, currentTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(controlHistOutVOList);
    }

    @AspectLog(description = "在途车辆", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "在途车辆", notes = "在途车辆", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/vehicleTraceHist",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject vehicleTrace(@RequestBody VehicleTraceInVO vehicleTraceInVO) throws ParseException {
        String adCode = vehicleTraceInVO.getAdCode();
        String currentTime = vehicleTraceInVO.getCurrentTime();
        List<VehicleTraceOutVO> vehicleTraceOutVOList = situationDetectionService.vehicleTrace(adCode, currentTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(vehicleTraceOutVOList);
    }

    @AspectLog(description = "图层-视频设备信息及状态", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "图层-视频设备信息及状态", notes = "图层-视频设备信息及状态", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/videoDeviceInfo",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject videoDeviceInfo(@RequestBody VideoDeviceInfoInVO videoDeviceInfoInVO) {
        String adCode = videoDeviceInfoInVO.getAdCode();
        List<VideoDeviceInfoOutVO> videoDeviceInfoOutVOList = situationDetectionService.videoDeviceInfo(adCode);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(videoDeviceInfoOutVOList);
    }

    @AspectLog(description = "绿波监测", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "绿波监测", notes = "绿波监测", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/greenwaveRealtime",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject greenwaveRealtime(@RequestBody GreenwaveRealtimeInVO greenwaveRealtimeInVO) throws ParseException {
        String adCode = greenwaveRealtimeInVO.getAdCode();
        String currentTime = greenwaveRealtimeInVO.getCurrentTime();
        List<GreenwaveRealtimeOutVO> greenwaveRealtimeOutVOList = situationDetectionService.greenwaveRealtime(adCode, currentTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(greenwaveRealtimeOutVOList);
    }

    @AspectLog(description = "交通设备状态", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "交通设备状态", notes = "交通设备状态", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/allDeviceStatus",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject allDeviceStatus(@RequestBody AllDeviceStatusInVO allDeviceStatusInVO) {
        String adCode = allDeviceStatusInVO.getAdCode();
        Map<String, Map<String, Integer>> allDeviceStatusMap = situationDetectionService.allDeviceStatus(adCode);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(allDeviceStatusMap);
    }

    @AspectLog(description = "绿波监测-关联路口", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "绿波监测-关联路口", notes = "绿波监测-关联路口", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/greenwaveCross",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject greenwaveCross(@RequestBody GreenwaveCrossInVO greenwaveCrossInVO) {
        String adCode = greenwaveCrossInVO.getAdCode();
        String currentTime = greenwaveCrossInVO.getCurrentTime();
        Integer id = greenwaveCrossInVO.getId();
        List<GreenwaveCrossOutVO> greenwaveCrossOutVOList = situationDetectionService.greenwaveCross(adCode, currentTime, id);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(greenwaveCrossOutVOList);
    }

    @AspectLog(description = "路口设备状态信息", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "路口设备状态信息", notes = "路口设备状态信息", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "crossDeviceStatusInfo", produces = MediaType.APPLICATION_JSON)
    public JsonViewObject crossDeviceStatusInfo() {
        List<CrossDeviceStatusInfoOutVO> list = situationDetectionService.crossDeviceStatusInfo();
        for (CrossDeviceStatusInfoOutVO item : list) {
            int status = item.getStatus();
            item.setStatus(status);
        }
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(list);
    }

    @AspectLog(description = "辖区分组", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "辖区分组", notes = "辖区分组")
    @GetMapping("/jurisdictionTree")
    public JsonViewObject jurisdictionTree(Integer areaId, Integer type) throws Exception {
        List<JurisdictionAreaTreeVO> result = situationDetectionService.jurisdictionTree(areaId, type);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(result);
    }

    @AspectLog(description = "查询区域列表", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "查询区域列表", notes = "查询区域列表")
    @GetMapping("/selectAreaList")
    public JsonViewObject selectAreaList(Integer areaId) {
        List<AreaListVO> areaListVOS = situationDetectionService.selectAreaList(areaId);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(areaListVOS);
    }

    @AspectLog(description = "查询信号机故障列表", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "查询信号机故障列表", notes = "查询信号机故障列表")
    @GetMapping("/listSignalFaultInfos")
    public JsonViewObject listSignalFaultInfos(String dealStatus) throws Exception {
        List<SignalFaultInfoVO> signalFaultInfoVOS = situationDetectionService.listSignalFaultInfos(dealStatus);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(signalFaultInfoVOS);
    }

    @AspectLog(description = "信号机故障统计", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "信号机故障统计", notes = "信号机故障统计")
    @GetMapping("/countSignalFaultInfos")
    public JsonViewObject countSignalFaultInfos() {
        DeviceFaultCountVO result = situationDetectionService.countSignalFaultInfos();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(result);
    }

    @AspectLog(description = "路口列表-手动控制", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "路口列表-手动控制", notes = "路口列表-手动控制")
    @GetMapping("/listSignalControlInfos")
    public JsonViewObject listSignalControlInfos(Integer operationMode) {
        List<SignalOperationModeVO> result = situationDetectionService.listSignalControlList(operationMode);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(result);
    }

    @AspectLog(description = "运行模式统计", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "运行模式统计", notes = "运行模式统计")
    @GetMapping("/countSignalControlInfos")
    public JsonViewObject countSignalControlInfos() {
        List<SignalOperationModeCountVO> result = situationDetectionService.countSignalControlInfos();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(result);
    }

    @AspectLog(description = "设备厂商", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "设备厂商", notes = "设备厂商")
    @GetMapping("/signalManufacturerInfoCount")
    public JsonViewObject signalManufacturerInfoCount() throws Exception {
        List<SignalManufacturerCountInfoVO> result = situationDetectionService.signalManufacturerInfoCount();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(result);
    }

}
