package net.wanji.web.controller.scheme;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.web.dto.RunningPlanDTO;
import net.wanji.web.service.scheme.impl.RunningPlanServiceImpl;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.text.ParseException;

/**
 * 方案管理-运行计划
 *
 * @author Kent HAN
 * @date 2022/12/20 10:14
 */
@Api(value = "RunningPlanController", description = "方案管理-运行计划")
@RequestMapping("/runningPlan")
@RestController
public class RunningPlanController {


    private final RunningPlanServiceImpl runningPlanService;

    public RunningPlanController(RunningPlanServiceImpl runningPlanService) {
        this.runningPlanService = runningPlanService;
    }

    @AspectLog(description = "保存运行计划", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "保存运行计划", notes = "保存运行计划", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/saveRunningPlan",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = JsonViewObject.class),
    })
    public JsonViewObject saveRunningPlan(@RequestBody RunningPlanDTO runningPlanDTO) throws ParseException {
        runningPlanService.saveRunningPlan(runningPlanDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @AspectLog(description = "运行计划列表", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "运行计划列表", notes = "运行计划列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/listRunningPlan",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningPlanDTO.class),
    })
    public JsonViewObject listRunningPlan(@RequestBody CrossIdDTO crossIdDTO) {
        RunningPlanDTO runningPlanDTO = runningPlanService.listRunningPlan(crossIdDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(runningPlanDTO);
    }
}
